const fs = require('fs');
const path = require('path');
const AWS = require('aws-sdk');
const { Client } = require('ssh2');
const axios = require('axios');
const downloadsPath = path.join(process.env.HOME || process.env.USERPROFILE, 'Downloads');
const rootKeyFilePath = path.join(downloadsPath, 'rootkey.csv');

// Check if the rootkey.csv file exists
if (!fs.existsSync(rootKeyFilePath)) {
    console.error(`File not found: ${rootKeyFilePath}`);
    process.exit(1);
}

// Function to read the second line and extract AWS credentials
function extractAWSCredentials(filePath) {
    const fileContent = fs.readFileSync(filePath, 'utf8');
    const lines = fileContent.split('\n');

    if (lines.length < 2) {
        throw new Error('The rootkey.csv file does not contain enough lines.');
    }

    const secondLine = lines[1].trim();
    const [accessKeyId, secretAccessKey] = secondLine.split(',');

    if (accessKeyId && secretAccessKey) {
        return {
            accessKeyId: accessKeyId.trim(),
            secretAccessKey: secretAccessKey.trim(),
        };
    } else {
        throw new Error('Failed to extract AWS credentials from the second line of rootkey.csv');
    }
}

// Function to create a single Lightsail instance
async function createLightsailInstance(lightsail, region, zone, instanceName) {
    const instanceParams = {
        blueprintId: 'ubuntu_20_04', // Ubuntu 20.04
        bundleId: 'large_2_0', // $44 USD plan for Lightsail
        availabilityZone: `${region}a`, // Availability zone (usually 'a', 'b', or 'c')
        instanceNames: [instanceName], // Only creating one instance at a time
    };

    try {
        const result = await lightsail.createInstances(instanceParams).promise();
        console.log(`Created instance ${instanceName} in region ${region}, zone ${zone}:`, result);
    } catch (error) {
        console.error(`Failed to create instance ${instanceName} in ${region}:`, error.message);
    }
}

// Function to add delay
const sleep = (ms) => new Promise((resolve) => setTimeout(resolve, ms));

async function main() {
    try {
        // Extract credentials
        const credentials = extractAWSCredentials(rootKeyFilePath);
        console.log('AWS Credentials Loaded:', credentials);

        // Set AWS config globally
        AWS.config.update({
            accessKeyId: credentials.accessKeyId,
            secretAccessKey: credentials.secretAccessKey,
        });

        // Define the regions and zones to create instances in
        const regionsAndZones = [


            { region: 'us-east-2', zone: 'Ohio', instanceName: 'Ubuntu-1' },
            { region: 'us-west-2', zone: 'Oregon', instanceName: 'Ubuntu-1' },
            { region: 'us-east-1', zone: 'Virginia', instanceName: 'Ubuntu-1' },
            { region: 'us-east-2', zone: 'Ohio', instanceName: 'Ubuntu-2' },
            { region: 'us-west-2', zone: 'Oregon', instanceName: 'Ubuntu-2' },
            { region: 'us-east-1', zone: 'Virginia', instanceName: 'Ubuntu-2' },
        ];

        // Loop through each region and create one instance at a time
        for (const { region, zone, instanceName } of regionsAndZones) {
            const lightsail = new AWS.Lightsail({ region });
            await createLightsailInstance(lightsail, region, zone, instanceName);
            console.log(`Waiting 30 seconds before creating the next instance...`);
            await sleep(60000); // Wait for 30 seconds
        }
        await sleep(60000);
        console.log('All Done')

    } catch (error) {
        console.error('Error:', error.message);
    }
}

main().catch(console.error);
